#-*- encoding:utf-8 -*-
'''
@author: guoliang@nfschina.com
'''


import pandas as pd
import os
import sys
from pathlib import Path
import glsu
import re

if sys.getdefaultencoding() != 'utf-8':
    reload(sys)
    sys.setdefaultencoding('utf-8')

def installFonts(files):
    res = glsu.checkSudo()
    if res:
        dest = '/usr/local/share/fonts/'
    else: 
        os.system('mkdir -p ~/.local/share/fonts/')
        dest = '~/.local/share/fonts/'   
    try:
        for file in files:
            temp_str = file.split("/")[-1]
            if None == re.match( "^[\w\.-]*$", temp_str, flags=0):
                return "Invalid file!", file
            cmd = 'cp %s %s' % (file, dest) 
            res = glsu.run(cmd)
            if res != 0:
                return res, cmd
        #-i before cd because cd can't be run directly in sudo
        #glsu.run(['mkfontscale', 'mkfontdir', 'fc-cache -fv'])
        cmd = ['mkfontscale', 'mkfontdir', 'fc-cache -fv', 'rm -f fontsdir', 'rm -f fonts.scale']
        res = glsu.run(cmd)
        return res, cmd        
    except Exception as e:
        return e, cmd
    
def installFontFiles(files):
    res = glsu.checkSudo()
    if res:
        dest = '/usr/local/share/fonts/'
    else: 
        os.system('mkdir -p ~/.local/share/fonts/')
        dest = '~/.local/share/fonts/'   
    try:
        cmd = 'cp %s %s' % (files, dest) 
        res = glsu.run(cmd)
        if res != 0:
            return res, cmd
        #-i before cd because cd can't be run directly in sudo
        #glsu.run(['mkfontscale', 'mkfontdir', 'fc-cache -fv'])
        cmd = ['mkfontscale', 'mkfontdir', 'fc-cache -fv', 'rm -f fontsdir', 'rm -f fonts.scale']
        res = glsu.run(cmd)
        return res, cmd        
    except Exception as e:
        return e, cmd

def uninstallFonts(files):
    try:
        for file in files:
            cmd = 'rm %s' % file
            user_home = os.path.expanduser('~')
            dest = user_home+'/.local/share/fonts/'
            #TY_os uninstall system fonts need use 'gksu' 
            if os.path.exists("/etc/.systeminfo") or file.startswith(dest) :
                res = glsu.run('rm %s' % file)
            else:
                res = glsu.run('pkexec rm %s' % file)
            if res!=0:
                return res, cmd
        cmd = 'fc-cache -fv'
        res = glsu.run(cmd)
        return res, cmd
    except Exception as e:
        return e, cmd

     
def listFc():
    # Execute the 'fc-list -v' command and get the output
    outputs = os.popen('fc-list -v').read()

    # Split the output into parts (each part represents a font)
    parts = outputs.split('\n\n')

    # Initialize a list for storing the font data
    fonts_data = []

    # Iterate through each part (each font's details)
    for part in parts:
        font = {}

        # Skip empty parts
        if part.strip() == '':
            continue

        # Try to process the part and extract information
        try:
            # Extract the basic font information (up to the 'charset:' line)
            up = part.split('\tcharset:')[0]
        except Exception as e:
            print(f"Error processing part: {e}")
            return pd.DataFrame()  # Return an empty DataFrame in case of error

        # Split into individual lines, skipping the first line (it's often just a header)
        lines = up.split('\n')[1:]

        # Process each line to extract key-value pairs
        for line in lines:
            words = line.split(':', 1)

            # Ensure there are at least two parts (key-value pair)
            if len(words) < 2:
                continue

            key = words[0].strip()
            value = words[1].strip()

            # Check if the key is one of the interested keys
            if key not in ['family', 'style', 'fullname', 'file']:
                continue

            # Clean up the value (remove any "(s)" or "(i)" suffixes)
            if value.endswith('(s)'):
                value = value[:-3]
            elif value.endswith('(i)'):
                value = value[:-3]

            # Remove surrounding quotes if present
            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]

            # Add key-value to the font dictionary
            font[key] = value

        # If 'family' is found, extract just the first part (before the first quote)
        font['family_'] = font.get('family', 'font without family').split('"')[0]

        # If 'style' is found, extract just the first part (before the first quote)
        font['style_'] = font.get('style', 'font without style').split('"')[0]

        # If 'fullname' is found, extract just the first part (before the first quote)
        font['fullname_'] = font.get('fullname', 'font without fullname').split('"')[0]

        # Check if the font file is a symbolic link
        font['is_symlink_'] = Path(font.get('file', '')).is_symlink()

        # Ensure the file path doesn't start with '/usr/share/fonts/truetype/'
        font['file_'] = font.get('file', '') if not font.get('file', '').startswith('/usr/share/fonts/truetype/') else font.get('file', '')[len('/usr/share/fonts/truetype/'):]

        # Append the font dictionary to the list
        fonts_data.append(font)

    # Convert the list of font dictionaries to a DataFrame
    df = pd.DataFrame(fonts_data)

    # Reset index for the final DataFrame
    df = df.reset_index(drop=True)

    return df



if __name__ == '__main__':
    df = listFc()
    #print(df[['family_', 'style', 'fullname', 'file']])
    #print(df[['family_', 'style_', 'fullname_', 'file']])
    df.to_csv('fonts.csv')
